@@ -3,6 +3,7 @@ package uk.ac.starlink.topcat.plot2;
 import java.io.IOException;
 import java.util.Arrays;
 import uk.ac.starlink.table.ColumnData;
+import uk.ac.starlink.table.DomainMapper;
 import uk.ac.starlink.table.RowSequence;
 import uk.ac.starlink.table.StarTable;
 import uk.ac.starlink.table.ValueInfo;
@@ -53,12 +54,29 @@ public class GuiDataSpec extends AbstractDataSpec {
         return contents_[ ic ].getCoord();
     }
 
-    public Object getCoordId( int ic ) {
-        return Arrays.asList( contents_[ ic ].getDataLabels() );
+    public String getCoordId( int ic ) {
+        String[] dataLabels = contents_[ ic ].getDataLabels();
+        DomainMapper[] dms = contents_[ ic ].getDomainMappers();
+        StringBuffer sbuf = new StringBuffer();
+        int nu = dataLabels.length;
+        for ( int iu = 0; iu < nu; iu++ ) {
+            sbuf.append( dataLabels[ iu ] );
+            DomainMapper dm = dms[ iu ];
+            if ( dm != null ) {
+                sbuf.append( "|" )
+                    .append( dm.getSourceName() );
+            }
+            sbuf.append( ";" );
+        }
+        return sbuf.toString();
     }
 
-    public Object getMaskId() {
-        return subset_;
+    public String getMaskId() {
+        return new StringBuffer()
+            .append( subset_.getClass().getName() )
+            .append( '@' )
+            .append( Integer.toHexString( System.identityHashCode( subset_ ) ) )
+            .toString();
     }
 
     public ValueInfo[] getUserCoordInfos( int ic ) {
@@ -71,6 +89,10 @@ public class GuiDataSpec extends AbstractDataSpec {
         return infos;
     }
 
+    public DomainMapper[] getUserCoordMappers( int ic ) {
+        return contents_[ ic ].getDomainMappers();
+    }
+
     public UserDataReader createUserDataReader() {
         int ncoord = contents_.length;
         final Object[][] userRows = new Object[ ncoord ][];
